#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Mentat::Message::Correlator::Context::Queue::Memory module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 48;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Mentat::Message::Correlator::Context::Queue::Memory';

        use_ok('Tester');
        use_ok('Mentat::Message::Correlator::Context');
        use_ok('Mentat::Message::Correlator::Context::Queue');
        use_ok($TESTED_CLASS);

        $ENV{PATH} = '/bin:/usr/bin';
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();
#Tester::DEBUG_ON(10);

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
### $Mentat::Message::Correlator::Context::Queue::Memory::VERSION

my $tmp;
my $c1 = Mentat::Message::Correlator::Context->new(id => 'id7', ttl => -15);
my $c2 = Mentat::Message::Correlator::Context->new(id => 'id6', ttl => -10);
my $c3 = Mentat::Message::Correlator::Context->new(id => 'id5', ttl => -5);
my $c4 = Mentat::Message::Correlator::Context->new(id => 'id4', ttl => 0);
my $c5 = Mentat::Message::Correlator::Context->new(id => 'id3', ttl => 5);
my $c6 = Mentat::Message::Correlator::Context->new(id => 'id2', ttl => 10);
my $c7 = Mentat::Message::Correlator::Context->new(id => 'id1', ttl => 15);

### Attempting to create Mentat::Message::Correlator::Context::Queue::Memory instance...
my $instance = $TESTED_CLASS->new();
isa_ok($instance, $TESTED_CLASS);

### Viewing the created instance: $instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D(2);

$instance->enqueue($c1);
$instance->enqueue($c2);
$instance->enqueue($c3);
$instance->enqueue($c4);
$instance->enqueue($c5);
$instance->enqueue($c6);
$instance->enqueue($c7);

is($instance->count, 7, "Count check test 01");

is($instance->exists('id1'), 1,  "Exists check test 01");
is($instance->exists('id2'), 1,  "Exists check test 02");
is($instance->exists('id3'), 1,  "Exists check test 03");
is($instance->exists('id4'), 1,  "Exists check test 04");
is($instance->exists('id5'), 1,  "Exists check test 05");
is($instance->exists('id6'), 1,  "Exists check test 06");
is($instance->exists('id7'), 1,  "Exists check test 07");
is($instance->exists('x'),   '', "Exists check test 08");

$tmp = $instance->dequeue;
is($tmp->id,                'id7', "Dequeue check test 01");
is($instance->count,         6,    "Count check test 02");
is($instance->count(1),      6,    "Count check test 03");
is($instance->exists('id7'), '',   "Exists check test 09");

$tmp = $instance->fetch('x');
is($tmp,                undef, "Fetch check test 01");
is($instance->count,    6,     "Count check test 04");
is($instance->count(1), 6,     "Count check test 05");

$tmp = $instance->fetch('id1');
is($tmp->id,                 'id1', "Fetch check test 02");
is($instance->count,         6,     "Count check test 06");
is($instance->count(1),      5,     "Count check test 07");
is($instance->exists('id1'), '',    "Exists check test 10");

$tmp->id('x');
$tmp->timeout(time-8);
$instance->enqueue($tmp);
is($instance->count,       7, "Count check test 08");
is($instance->count(1),    6, "Count check test 09");
is($instance->exists('x'), 1, "Exists check test 11");

is($instance->dequeue->id, 'id6', "Dequeue check test 02");
is($instance->dequeue->id, 'x',   "Dequeue check test 03");
is($instance->dequeue->id, 'id5', "Dequeue check test 04");
is($instance->dequeue->id, 'id4', "Dequeue check test 05");
is($instance->dequeue,     undef, "Dequeue check test 06");
is($instance->count,       3,     "Count check test 10");
is($instance->count(1),    2,     "Count check test 11");

is($instance->exists('id1'), '', "Exists check test 12");
is($instance->exists('id2'), 1,  "Exists check test 13");
is($instance->exists('id3'), 1,  "Exists check test 14");
is($instance->exists('id4'), '', "Exists check test 15");
is($instance->exists('id5'), '', "Exists check test 16");
is($instance->exists('id6'), '', "Exists check test 17");
is($instance->exists('id7'), '', "Exists check test 18");
is($instance->exists('x'),   '', "Exists check test 19");

is($instance->fetch('id2')->id, 'id2', "Fetch check test 02");
is($instance->count,            3,     "Count check test 12");
is($instance->count(1),         1,     "Count check test 13");
is($instance->exists('id2'),    '',    "Exists check test 20");
is($instance->exists('id3'),    1,     "Exists check test 21");
